#!/bin/bash
#
# Simple script to add/remove Tomcat instances.
# Part of tomcat6-addons package.

# Include settings from headerfile.
HEADER="/etc/default/tomcat6-addons"
if [ ! -f "$HEADER" ]; then echo -e "Missing config file: $HEADER - stopped." >&2; exit 1; fi
. $HEADER

# Root privileges required.
if [ ! $UID -eq 0 ]; then
    echo -e "$MSG_ROOTNEED" >&2; exit $EXIT_ERR
fi

#
# Functions
#

# tomcat_configure [force]
# Graceful setup or reconfiguration of multiple Tomcat environment.
#
# * Installs Tomcat via apt-get.
# * Creates multiplex Tomcat environment.
# * Transform installed Tomcat to "system" Tomcat instance.
# * Configures MySQL JDBC.
# * Installs and configures Apache-Tomcat connector.
# * Installs and configures JavaMail.
#
# Shows greeting and gets confirmation except calling with 'force'.
#
function tomcat_configure {

    local result
    # Creating timestamp to name backup files and logfile.
    local timestamp="$($DATE '+%Y%m%d%H%M%S')"
    # Shows greeting message and get confirmed.
    local REPLY
    if [ "$1" != "force" ]; then
	echo -e "$MSG_GREETING"
        echo; read -s -n1 -p "$MSG_CONFIRM"; echo; echo
        if [ "$REPLY" != "y" -a "$REPLY" != "Y" ]; then return $EXIT_ERR; fi
    else
	shift
    fi
    # Let's go!

    # Install tomcat6 packages if isn't installed. Stops tomcat6.
    log "$MSG_INSTALLTOMCAT"
    if [ -f "$TOMCAT_INITSCRIPT" ]; then
	log "$TOMCAT_NAME $MSG_ALREADY_INSTALLED"
    else
	# Try to install necessary packeges, giving up on error.
	log "$($APT_GET_INSTALL -q -y $TOMCAT_PACKAGES)"
	# Giving up on error.
	if [ -f "$TOMCAT_INITSCRIPT" ]; then
	    log "$MSG_FAILED_INSTALL $TOMCAT_PACKAGES"
	    return $EXIT_ERR
	fi
    fi
    # Maybe we need to write /usr.
    log $($MOUNT -v -o remount,rw /usr)
    # Halt maybe unconfigured Tomcat.
    log $($TOMCAT_INITSCRIPT stop 2>&1)

    # Add 'system' instance with default parameters.
    # Does nothing if already configured.
    tomcat_add "$TOMCAT_SYSTEMNAME" "$TOMCAT_DEFSTANDALONEPORT" "$TOMCAT_DEFUSER" "$TOMCAT_DEFGROUP"; result=$?
    # Giving up on error.
    if [ "$result" -ne 0 ]; then return $result; fi
    # Activating 'system' instance in /etc/default.
    if [ -e "$TOMCAT_DEFAULTDIR/$TOMCAT_NAME" ]; then
	if [ ! -L "$TOMCAT_DEFAULTDIR/$TOMCAT_NAME" ]; then
	    log $(mv -v "$TOMCAT_DEFAULTDIR/$TOMCAT_NAME" "$TOMCAT_DEFAULTDIR/$TOMCAT_NAME.$timestamp")
	else
	    rm "$TOMCAT_DEFAULTDIR/$TOMCAT_NAME" 2>&1 >/dev/null
	fi
    fi
    log $(ln -v -s "$TOMCAT_NAME.$TOMCAT_SYSTEMNAME" "$TOMCAT_DEFAULTDIR/$TOMCAT_NAME")

    # Installing and configuring Apache-Tomcat connector.
    log "$MSG_INSTALL_APACHETOMCAT"
    if [ ! -e "$APACHE_TOMCAT_MODULE" ]; then
	log "$($APT_GET_INSTALL -q -y $APACHE_TOMCAT_PACKAGES)"
	# We need write to /usr.
	log $($MOUNT -v -o remount,rw /usr)
	if [ ! -e "$APACHE_TOMCAT_MODULE" ]; then
	    log "$MSG_FAILED_INSTALL $APACHE_TOMCAT_PACKAGES"
	fi
    else
	log "$APACHE_TOMCAT_MODULE $MSG_ALREADY_INSTALLED"
    fi
    # Configuration.
    if [ -e "$APACHE_TOMCAT_MODULE" ]; then
	# Apache-side configuration.
	if [ ! -f "$APACHE_CONFIGDIR/$APACHE_TOMCAT_ACONF" ]; then
	    export APACHE_TOMCAT_CONFIGDIR APACHE_TOMCAT_LOGFILE APACHE_TOMCAT_TCONF \
	           APACHE_TOMCAT_JKSHMFILE
	    $CAT $INSTALL_TEMPLATES/$APACHE_TOMCAT_ACONF | $ENVSUBST > $APACHE_CONFIGDIR/$APACHE_TOMCAT_ACONF
	    $CHMOD 644 $APACHE_CONFIGDIR/$APACHE_TOMCAT_ACONF
	    log "$APACHE_CONFIGDIR/$APACHE_TOMCAT_ACONF $MSG_CONFIGURED"
	else
	    log "$APACHE_CONFIGDIR/$APACHE_TOMCAT_ACONF $MSG_ALREADY_CONFIGURED"
	fi
	# Tomcat-side configuration for system Tomcat.
	# Save maintainer's configuration if isn't saved.
	if [ ! -f "$APACHE_TOMCAT_CONFIGDIR/$APACHE_TOMCAT_TCONF.dpkg" ]; then
	    if [ -f "$APACHE_TOMCAT_CONFIGDIR/$APACHE_TOMCAT_TCONF" ]; then
		log $($MV -v $APACHE_TOMCAT_CONFIGDIR/$APACHE_TOMCAT_TCONF $APACHE_TOMCAT_CONFIGDIR/$APACHE_TOMCAT_TCONF.dpkg)
	    fi
	fi
	# Creating configuration but not overwriting existing one.
	if [ ! -f "$APACHE_TOMCAT_CONFIGDIR/$APACHE_TOMCAT_TCONF" ]; then
	    export JAVA_HOME TOMCAT_DEFAJPPORT TOMCAT_SYSTEMNAME
	    $CAT $INSTALL_TEMPLATES/$APACHE_TOMCAT_TCONF | $ENVSUBST > $APACHE_TOMCAT_CONFIGDIR/$APACHE_TOMCAT_TCONF
	    $CHMOD 644 $APACHE_TOMCAT_CONFIGDIR/$APACHE_TOMCAT_TCONF
	    log "$APACHE_TOMCAT_CONFIGDIR/$APACHE_TOMCAT_TCONF $MSG_CONFIGURED"
	else
	    log "$APACHE_TOMCAT_CONFIGDIR/$APACHE_TOMCAT_TCONF $MSG_ALREADY_CONFIGURED"
	fi
    fi

    # Installing JavaMail.
    log "$MSG_INSTALLJAVAMAIL"
    if [ ! -e $JAVAMAIL_JAR ]; then
	# Try to install from Debian packages.
	log "$($APT_GET_INSTALL -q -y $JAVAMAIL_PACKAGES)"
	# We need write to /usr.
	log $($MOUNT -v -o remount,rw /usr)
	if [ ! -e $JAVAMAIL_JAR ]; then
	    log "$MSG_FAILED_INSTALL $JAVAMAIL_PACKAGES"
	fi
    else
	log "$JAVAMAIL_JAR $MSG_ALREADY_INSTALLED"
    fi
    # Try to create symlink pointing to JavaMail.
    if [ ! -L "$JAVAMAIL_LINK" ]; then
	if [ -e "$JAVAMAIL_JAR" ]; then
	    log $($LN -v -s $JAVAMAIL_JAR $JAVAMAIL_LINK)
	else
	    log "$MSG_MISSING $JAVAMAIL_JAR"
	fi
    else
	log "$JAVAMAIL_LINK $MSG_ALREADY_INSTALLED"
    fi

    # Configuring MySQL JDBC driver as Tomcat library if already installed.
    log "$MSG_INSTALLJDBC"
    # Try to create symlink pointing to MySQL JDBC Connector.
    if [ ! -L "$MYSQL_JDBC_LINK" ]; then
	if [ -f "$MYSQL_JDBC" ]; then
	    log $($LN -v -s $MYSQL_JDBC $MYSQL_JDBC_LINK)
	else
	    log "$MSG_MISSING $MYSQL_JDBC"
	fi
    else
	log "$MYSQL_JDBC_LINK $MSG_ALREADY_INSTALLED"
    fi

    # Finished - restoring /usr.
    log $($MOUNT -v -o remount /usr)
    log "$MSG_FINISHED"
    echo -e "\n$MSG_GOODBYE"
    return $result
}

# tomcat_add [[[[ instance_name(=system) ] service_port(=8080) ] Linux_user(=www-data) ] Linux_group(=www-data) ]
# Add a new Tomcat instance in /var/opt/tomcat6/instance_name (as CATALINA_BASE).
# Install host-manager, manager and tomcat-docs webapplications at localhost.
#
# Doesn't hurts an existing instance.
#
function tomcat_add {

    local result
    # Getting parameters.
    # Instance name (defaults to system). Alphanumeric only.
    local instancename=${1:-$TOMCAT_SYSTEMNAME}; shift
    if [ ! -z "`echo "$instancename" | $SED "s/$MASKINSTANCE//"`" ]; then 
	log "$MSG_WRONG_PARAMETER $instancename"; return $EXIT_ERR; fi
    # Service port (defaults to 8080). Valid numbers: 8080-8109.
    # Shutdown, redirect and AJP ports calculated also.
    local standaloneport=${1:-$TOMCAT_DEFSTANDALONEPORT}; shift
    if [ ! -z "`echo "$standaloneport" | $SED "s/$MASKPORT//"`" ]; then
	log "$MSG_WRONG_PARAMETER $standaloneport"; return $EXIT_ERR; fi
    if [ "$standaloneport" -lt 8080 -o "$standaloneport" -ge 8110 ]; then 
	log "$MSG_WRONG_PARAMETER $standaloneport"; return $EXIT_ERR; fi
    local port_diff=$(( standaloneport - TOMCAT_DEFSTANDALONEPORT ))
    local redirectport=$(( TOMCAT_DEFREDIRECTPORT + port_diff ))
    local shutdownport=$(( TOMCAT_DEFSHUTDOWNPORT + port_diff ))
    local ajpport=$(( TOMCAT_DEFAJPPORT + port_diff ))
    # Linux user running this instance (defaults to tomcat6). Must exist!
    local linuxuser=${1:-$TOMCAT_DEFUSER}; shift
    if [ -z "$($GETENT passwd | $GREP "^$linuxuser:")" ]; then
	log "$MSG_WRONG_PARAMETER $linuxuser"; return $EXIT_ERR; fi
    # Linux group running this instance (defaults to tomcat6). Must exist!
    local linuxgroup=${1:-$TOMCAT_DEFGROUP}; shift
    if [ -z "$($GETENT group | $GREP "^$linuxgroup:")" ]; then
	log "$MSG_WRONG_PARAMETER $linuxgroup"; return $EXIT_ERR; fi
    # Debug logging parameters.
    log "Parameters: $instancename $standaloneport $redirectport $shutdownport $ajpport $linuxuser $linuxgroup"

    # Creating CATALINA_BASE folder structure from skeleton.
    # Checking multitomcat root folder, creating if necessary.
    if [ ! -d "$TOMCAT_BASEDIR" ]; then log $($MKDIR -v -m 755 $TOMCAT_BASEDIR); fi
    # Checking instance's root folder, leaving alone if existing.
    if [ ! -d "$TOMCAT_BASEDIR/$instancename" ]; then
	# Creating if necessary, try to force o-rwx priviledge.
	log $($MKDIR -v -m 750 $TOMCAT_BASEDIR/$instancename)
	$SETFACL -d -m o::--- $TOMCAT_BASEDIR/$instancename
	# Filling with skeleton - resolving symlinks and copying content itself.
	$CP -rL --preserve $INSTALL_SKELETON/* $TOMCAT_BASEDIR/$instancename/
	# Setting ownership and permissions.
	$CHOWN -R $linuxuser:$linuxgroup $TOMCAT_BASEDIR/$instancename
	$CHMOD -R o-rwx $TOMCAT_BASEDIR/$instancename

	# Customizing Catalina configuration.
	# Originals saved with .bak extension during skeleton copy.
	local TOMCAT_CONFIGDIR="$TOMCAT_BASEDIR/$instancename/conf"
	# Creating tomcat-users.xml from template.
	local PASSWORD=`$MAKEPASSWD`; log "$MSG_MANAGERS_PW_IS $PASSWORD"
	export PASSWORD
	$CAT $INSTALL_TEMPLATES/$TOMCAT_USERS_XML | $ENVSUBST > \
             $TOMCAT_CONFIGDIR/$TOMCAT_USERS_XML
	$CHOWN $linuxuser:$linuxgroup $TOMCAT_CONFIGDIR/$TOMCAT_USERS_XML
	$CHMOD o-rwx $TOMCAT_CONFIGDIR/$TOMCAT_USERS_XML
	# Creating server.xml - setting ports, removing comments and empty lines.
	export ajpport redirectport shutdownport standaloneport
	$CAT $INSTALL_TEMPLATES/$TOMCAT_SERVER_XML | $ENVSUBST | \
	     $AWK '/<!--/ {off=1} /-->/ {off=2} /([\s\S]*)/ {if (off==0) print; if (off==2) off=0}' | \
	     $GREP -v "^[[:space:]]*$" \
             > $TOMCAT_CONFIGDIR/$TOMCAT_SERVER_XML
	$CHOWN $linuxuser:$linuxgroup $TOMCAT_CONFIGDIR/$TOMCAT_SERVER_XML
	$CHMOD o-rwx $TOMCAT_CONFIGDIR/$TOMCAT_SERVER_XML
	log "$TOMCAT_BASEDIR/$instancename $MSG_CONFIGURED"
    else
	log "$TOMCAT_BASEDIR/$instancename $MSG_ALREADY_INSTALLED"
    fi
    # Done with CATALINA_BASE.

    # Writing /etc/default file for instance if isn't exists.
    if [ ! -f "$TOMCAT_DEFAULTDIR/$TOMCAT_NAME.$instancename" ]; then
	export CATALINA_HOME JAVA_HOME JAVA_OPTS JAVA_TMP TOMCAT_BASEDIR TOMCAT_KEEPLOGS \
	       TOMCAT_NAME TOMCAT_SECURITY TOMCAT_SHUTDOWN_TIME TOMCAT_USER \
	       instancename linuxgroup linuxuser
	$CAT $INSTALL_TEMPLATES/$TOMCAT_TEMPLATE | $ENVSUBST > $TOMCAT_DEFAULTDIR/$TOMCAT_NAME.$instancename
	$CHMOD 644 $TOMCAT_DEFAULTDIR/$TOMCAT_NAME.$instancename
	log "$TOMCAT_DEFAULTDIR/$TOMCAT_NAME.$instancename $MSG_CONFIGURED"
    else
	log "$TOMCAT_DEFAULTDIR/$TOMCAT_NAME.$instancename $MSG_ALREADY_CONFIGURED"
    fi
    # Done with /etc/default file.

    # Rotating catalina.out and garbage collector logs.
    if [ ! -f "$LOGROTATE_ROOTDIR/$TOMCAT_NAME-$instancename" ]; then
	export TOMCAT_BASEDIR TOMCAT_STARTSTOP instancename linuxgroup linuxuser
	$CAT $INSTALL_TEMPLATES/$LOGROTATE_TEMPLATE | $ENVSUBST \
	     > $LOGROTATE_ROOTDIR/$TOMCAT_NAME-$instancename
	log "$LOGROTATE_ROOTDIR/$TOMCAT_NAME-$instancename $MSG_CONFIGURED"
    else
	log "$LOGROTATE_ROOTDIR/$TOMCAT_NAME-$instancename $MSG_ALREADY_INSTALLED"
    fi

    # Tomcat instance added.
    return $result
}

function tomcat_createweb {

    local result

    log "Under construction..."
    return $result
}

function tomcat_remove {

    local result

    log "Under construction..."
    return $result
}

function tomcat_removeweb {

    local result

    log "Under construction..."
    return $result
}

function tomcat_saveweb {

    local result

    log "Under construction..."
    return $result
}

# Helper function: logs with timestamp into a logfile and native to STD_ERR.
function log {

    local DATETIME="+%Y.%m.%d. %H:%M:%S"
    if [ -f "$TOMCAT_LOGFILE" -a -w "$TOMCAT_LOGFILE" ]; then
        echo -e "$($DATE "$DATETIME") $($WHOAMI) ${FUNCNAME[1]} $*" >>$TOMCAT_LOGFILE
        echo -e "$*" >&2
    else
        echo -e "$($DATE "$DATETIME") $($WHOAMI) ${FUNCNAME[1]} $*" >&2
    fi
    return
}

#
# Main program
#

# Logs handling: tomcat-manager log creation (rotated via Catalina cronjob).
# Must be the 1st step to enable logging all activities.
if [ ! -f "$TOMCAT_LOGFILE" ]; then
    touch $TOMCAT_LOGFILE; chown root:root $TOMCAT_LOGFILE; chmod 640 $TOMCAT_LOGFILE
    log "$MSG_LOGFILE_CREATED" "$TOMCAT_LOGFILE"
fi

# Resolving options, calling activities.
option=`echo $1 | $SED -n "s/^-\+//p"`; if [ ! -z "$option" ]; then shift; fi
case "$option" in
    "a"|"add")
        tomcat_add $*; exit $?
    ;;
    "configure"|"install")
        tomcat_configure $*; exit $?
    ;;
    "cw"|"createweb")
	tomcat_createweb $*; exit $?
    ;;
    "d"|"r"|"delete"|"remove")
        tomcat_remove $*; exit $?
    ;;
    "dw"|"rw"|"deleteweb"|"removeweb")
        tomcat_removeweb $*; exit $?
    ;;
    "sw"|"saveweb")
	tomcat_saveweb $*; exit $?
    ;;
    *)
        echo -e "$USAGE"; exit $EXIT_ERR
    ;;
esac

# That's all, folks! :-)
exit $EXIT_SUCC
