#!/bin/bash
#
# Simple wrapper to standard /etc/init.d/tomcat6 initscript.
# Modifies /etc/default/tomcat6 symlink before calling them.
# Part of tomcat6-addons package.

AWK="/usr/bin/awk"			# awk command call
DEFAULT="/etc/default"			# Defaults folder
EXIT_ERR=1				# Exit code on error
GREP="/bin/grep"			# grep command call
PS="/bin/ps"				# ps command call
SED="/bin/sed"				# sed command call
SLEEPTIME="1"				# Delay between actions (secs)
TOMCAT="tomcat6"			# Tomcat's name
TOMCAT_INITSCRIPT="/etc/init.d/tomcat6"	# Standard init script for Tomcat
TOMCAT_LIST=`ls -1 $DEFAULT/$TOMCAT.* 2>/dev/null | $SED "s/^.*\/\(.*\)$/\1/"`
TOMCAT_NAMES=$(echo -e "$TOMCAT_LIST" | $SED "s/^$TOMCAT\.//" | $GREP -v "^[0-9]*$")
WC="/usr/bin/wc"			# wc command call

MSG_ALREADY_RUNNING="Another instance is running - aborted."
MSG_ROOTNEED="Run as root, please."
MSG_NOTSYMLINK="Isn't a symlink:"
USAGE="Usage: $0 [-a|--all|-l|--list| instance_name] command(s)"

# Function to get PIDS of maybe another instances of this script.
# Gets a process list with same basename, ignores any processes
# related to this (parents, childs) and lists remaining.
#
function show_samename_pids {

    # Utilities used.
#   local AWK=${AWK:-$which awk}        # awk  command call
#   local GREP=${GREP:-$which grep}     # grep command call
#   local PS=${PS:-$which ps}           # ps   command call
#   local SED=${SED:-$which sed}        # sed  command call
#   local WC=${WC:-$which wc}           # wc   command call

    local PROCNAME=$0
    local MYPID=$$
    local PROCESSES=$($PS ahwx -f)
    # Gets related PIDs from PS output with AWK script.
    # From: http://www.tek-tips.com/faqs.cfm?fid=4602
    local RELPIDS=$(echo -e "$PROCESSES" | $AWK -v pid=$MYPID '
    BEGIN{if(!pid){
     t="/dev/tty";printf "PID ? ">t;getline<t;pid+=$1
    }}
    function child(id  ,i){
     print line[id];if(id==pid)printf "\n"
     for(i in ppid) if(ppid[i]==id) child(i)
    }
    function parent(id  ,i){
     i=ppid[id]
     if(line[i]>"" && i!=id) buf=parent(i)"\n"line[i]
     return buf
    }
    $2==pid{++found}
    {ppid[$2]=$3;line[$2]=$0}
    END{if(found){
      print line["PID"]"\n"parent(pid)"\n";child(pid)
    }}
    ' | $AWK '{print $2}')
    # Compare related PIDS list (to ignore) and PIDS with same command name.
    # Print all not related PIDs.
    for check_pid in $(echo "$PROCESSES" | $GREP "$PROCNAME " | $AWK '{print $2}')
    do
        for ignore_pid in $RELPIDS
        do
            if [ $check_pid -eq $ignore_pid ]; then continue 2; fi
        done
        echo -ne "$check_pid "
    done
    # Returns a space-delimited PID list.
}

#
# Main program
#

# Getting one option (if any).
option=`echo $1 | $SED -n "s/^-\+//p"`; if [ ! -z "$option" ]; then shift; fi
case "$option" in
    "l"|"list")
	# Shows all living multitomcat names (w/o backups) and finish.
	echo "$TOMCAT_NAMES"; exit 0
    ;;
    "a"|"all")
	# Root privileges required.
	if [ ! $UID -eq 0 ]; then
    	    echo -e "$MSG_ROOTNEED" >&2; exit $EXIT_ERR
	fi
	# Check another instance of this script running.
	if [ ! -z "$(show_samename_pids)" ]; then
	    echo -e "$MSG_ALREADY_RUNNING"; exit $EXIT_ERR
	fi
	# Enumerates Tomcat instances, redirects /etc/default/tomcat6 symlink
	# and calls standard init script with command.
	if [ -L "$DEFAULT/$TOMCAT" ]; then
	    for current_tomcat in $TOMCAT_NAMES
	    do
		rm $DEFAULT/$TOMCAT
		ln -s $TOMCAT.$current_tomcat $DEFAULT/$TOMCAT
		$TOMCAT_INITSCRIPT $*
		sleep $SLEEPTIME
	    done
	else
	    # If /etc/default/tomcat6 isn't a symlink, simply calls init script.
	    # Silently does nothing if /etc/default/tomcat6 totally missing.
	    if [ -e "$DEFAULT/$TOMCAT" ]; then
		$TOMCAT_INITSCRIPT $*
	    fi
	fi
    ;;
    *)
	# Root privileges required.
	if [ ! $UID -eq 0 ]; then
    	    echo -e "$MSG_ROOTNEED" >&2; exit $EXIT_ERR
	fi
	# Check another instance of this script running.
	if [ ! -z "$(show_samename_pids)" ]; then
	    echo -e "$MSG_ALREADY_RUNNING"; exit $EXIT_ERR
	fi
	# 1st parameter expected as a valid Tomcat's name - checking.
	selected_tomcat=$1; shift
	if [ -z "$(echo -e "$TOMCAT_NAMES" | $SED -n "s/^$selected_tomcat$/X/p")" ]; then
	    # On failure giving up with error message.
	    echo -e "$USAGE\n"; exit $EXIT_ERR
	else
	    # Some safety - if /etc/default/tomcat6 missing, creates them.
	    if [ ! -e "$DEFAULT/$TOMCAT" ]; then
		ln -s $TOMCAT.$selected_tomcat $DEFAULT/$TOMCAT
	    fi
	    # If /etc/default/tomcat6 is a symlink, changes it and calls the
	    # standard initscript.
	    if [ -L "$DEFAULT/$TOMCAT" ]; then
		rm $DEFAULT/$TOMCAT
		ln -s $TOMCAT.$selected_tomcat $DEFAULT/$TOMCAT
		$TOMCAT_INITSCRIPT $*
		exit $?
	    else
		# Otherwise something went wrong...
		echo -e "$MSG_NOTSYMLINK $DEFAULT/$TOMCAT"
		exit $EXIT_ERR
	    fi
	fi
    ;;
esac

# That's all, folks :-)
